Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

Imports PortSet

Namespace SimpleTerm
	Public Partial Class Form1
		Inherits Form
		Public Sub New()
			InitializeComponent()

			' show the settings in the form text line
			showSettings()
		End Sub

		Private Sub settingsToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles settingsToolStripMenuItem.Click
			' Make sure the port isn't already open
			If serialPort1.IsOpen Then
				MessageBox.Show("The port must be closed before changing the settings.")
				Return
			Else
				' creat an instance of the settings form
				Dim settings As PortSettings = New PortSettings()

				If settings.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
					If settings.selectedPort <> "" Then
						' set the serial port to the new settings
						serialPort1.PortName = settings.selectedPort
						serialPort1.BaudRate = settings.selectedBaudrate
						serialPort1.DataBits = settings.selectedDataBits
						serialPort1.Parity = settings.selectedParity
						serialPort1.StopBits = settings.selectedStopBits

						' show the new settings in the form text line
						showSettings()
					Else
						MessageBox.Show("Error: Settings form returned with no COM port selected.")
						Return ' bail out
					End If

				Else
					MessageBox.Show("Error: buttonSetup_Click - Settings dialog box did not return Okay.")
					Return ' bail out
				End If

				' open the port
				Try
					serialPort1.Close()
					serialPort1.Open()
					menuStrip1.Items(1).Text = "Close Port"

					showSettings()
				Catch ex As System.Exception
                    MessageBox.Show("Error - setupToolStripMenuItem_Click Exception: ", ex.ToString())
				End Try
			End If
		End Sub

		' show the settings in the form text line
		Private Sub showSettings()
			Me.Text = "Smiley Micros - " & serialPort1.PortName & " " & serialPort1.BaudRate.ToString() & "," & serialPort1.Parity & "," & serialPort1.DataBits.ToString() & "," & serialPort1.StopBits
			If serialPort1.IsOpen Then
				Me.Text &= " - Port is open"
			Else
				Me.Text &= " - Port is closed"
			End If
		End Sub

		' toggle the port state
		Private Sub openPortToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles openPortToolStripMenuItem.Click
			Try
				If serialPort1.IsOpen Then
					serialPort1.Close()
					menuStrip1.Items(1).Text = "Open Port"
				Else
					serialPort1.Open()
					menuStrip1.Items(1).Text = "Close Port"
				End If

				showSettings()
			Catch ex As System.Exception
                MessageBox.Show("Error - openPortToolStripMenuItem_Click Exception: ", ex.ToString())
			End Try
		End Sub

		Private Sub clearToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles clearToolStripMenuItem.Click
			richTextBox1.Text = ""
			richTextBox2.Text = ""
		End Sub

		#Region "transmit functions"

		Private Sub richTextBox1_KeyPress(ByVal sender As Object, ByVal e As KeyPressEventArgs) Handles richTextBox1.KeyPress
			sendChar(e.KeyChar)
		End Sub

		Private Sub sendChar(ByVal c As Char)
			Dim data As Char() = New Char(0){}
			data(0) = c
			Try
				serialPort1.Write(data, 0, 1)
			Catch
				MessageBox.Show("Error: sendByte - failed to send." & Constants.vbLf & "Is the port open?")
			End Try
		End Sub

		#End Region

		#Region "receive functions"

		' we want to have the serial port thread report back data received, but to display
		' that data we must create a delegate function to show the data in the richTextBox

		' define the delegate 
		Public Delegate Sub SetText()
		' define an instance of the delegate
        Private setTextI As SetText

		' create a string that will be loaded with the data received from the port
		Public str As String = ""

		' note that this function runs in a separate thread and thus we must use a delegate in order
		' to display the results in the richTextBox.
		Private Sub serialPort1_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles serialPort1.DataReceived
			' instantiate the delegate to be invoked by this thread
            setTextI = New SetText(AddressOf mySetText)

			' load the data into the string
			Try
				str = serialPort1.ReadExisting()
			Catch ex As System.Exception
                MessageBox.Show("Error - port_DataReceived Exception: ", ex.ToString())
			End Try

			' invoke the delegate in the MainForm thread
            Me.Invoke(setTextI)
		End Sub

		' create the instance of the delegate to be used to write the received data to the richTextBox
		Public Sub mySetText()
			' show the text
			richTextBox2.Text += str.ToString()

			moveCaretToEnd()
		End Sub

		' This rigaramole is needed to keep the last received item displayed
		' it kind of flickers and should be fixed
		Private Sub richTextBoxReceive_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs)
			moveCaretToEnd()
		End Sub

		Private Sub moveCaretToEnd()
			richTextBox1.SelectionStart = richTextBox1.Text.Length
			richTextBox1.SelectionLength = 0
			richTextBox1.ScrollToCaret()
		End Sub

		#End Region


	End Class
End Namespace


